package org.fortiss.tooling.kernel.ui.service;

import java.util.Set;

import org.fortiss.tooling.kernel.model.constraints.IConstraint;
import org.fortiss.tooling.kernel.model.constraints.IConstraintContainer;
import org.fortiss.tooling.kernel.model.constraints.IConstraintVerificationStatus;
import org.fortiss.tooling.kernel.service.IConstraintVerificationService;
import org.fortiss.tooling.kernel.ui.extension.IConstraintVerifierUI;
import org.fortiss.tooling.kernel.ui.internal.ConstraintVerificationUIService;

/**
 * GUI aspects of {@link IConstraintVerificationService}.
 * 
 * @author aravantinos
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 088353619375F645ACA70AD1C2990CE1
 */
public interface IConstraintVerificationUIService {
	/** Returns the service instance. */
	public static IConstraintVerificationUIService getInstance() {
		return ConstraintVerificationUIService.getInstance();
	}

	/** Action to take when trying to open the (status of the given) constraint. */
	void openStatus(IConstraint c);

	/** True if the (status of the) the given constraint can be open. */
	boolean canOpen(IConstraint c);

	/** Install the given constraint: install notifiers and triggers an "onOutdate" event. */
	void install(IConstraint constraint);

	/** Registers the verifier UI part with the kernel. */
	<T extends IConstraint> void registerConstraintVerifierUI(IConstraintVerifierUI<T> verifier,
			Class<T> constraintElementClass);

	/**
	 * The following features deal with the usage of constraints to drive the process.
	 * 
	 * EXPERIMENTAL.
	 */

	/**
	 * Returns the status of the constraint, possibly returns <code>null</code> if the constraint is
	 * not activated.
	 */
	IConstraintVerificationStatus getStatus(IConstraint constraint);

	/** Returns a list of all available constraint types. */
	Set<Class<IConstraint>> getAvailableConstraintTypes();

	/** Activates the constraint type in the context of the given constraints container. */
	<T extends IConstraint> void activate(Class<T> constraintType,
			IConstraintContainer cstrContainer);

	/** Deactivates the constraint type in the context of the given constraints container. */
	<T extends IConstraint> void deactivate(Class<T> constraintType,
			IConstraintContainer cstrContainer);

	/** Returns a description for the given constraint type. */
	<T extends IConstraint> String getDescription(Class<T> constraintType);
}
